/*
 * carl9170 firmware - used by the ar9170 wireless device
 *
 * Firmware context definition
 *
 * Copyright	2009	Christian Lamparter <chunkeey@googlemail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef __AR9170_H
#define __AR9170_H

#include "version.h"
#include "config.h"
#include "types.h"
#include "compiler.h"
#include "linux/fwcmd.h"
#include "linux/hw.h"
#include "desc.h"
#include "usb.h"
#include "cmd.h"

struct carl9170_bar_ctx {
	uint8_t ta[6];
	uint8_t ra[6];
	__le16 start_seq_num;
	__le16 control;
	__le32 phy;
};

struct firmware_context_struct {
	/* timer / clocks */
	uint32_t bogoclock;			/* supposed to be CPU clock in KHz */
	uint32_t counter;			/* main() cycles */

	/* misc */
	uint8_t watchdog_enable;

	struct {
		/* Host Interface DMA queues */
		struct dma_queue up_queue;	/* used to send frames to the host */
		struct dma_queue down_queue;	/* stores incoming frames from the host */
	} pta;

	struct {
		/* Hardware DMA queues */
		struct dma_queue tx_queue[__AR9170_NUM_TX_QUEUES];	/* wlan tx queue */
		struct dma_queue rx_queue;				/* wlan rx queue */

#ifdef CONFIG_CARL9170FW_DELAYED_TX
		struct dma_queue tx_delay[__AR9170_NUM_TX_QUEUES];
		struct dma_queue tx_retry;
		uint32_t tx_trigger;
#endif /* CONFIG_CARL9170FW_DELAYED_TX */

		/* Hardware DMA queue unstuck/fix detection */
		uint8_t last_tx_desc_num[__AR9170_NUM_TX_QUEUES];
		struct dma_desc *last_tx_desc[__AR9170_NUM_TX_QUEUES];
		uint32_t rx_total;
		uint32_t rx_overruns;
		bool mac_reset;

#ifdef CONFIG_CARL9170FW_CAB_QUEUE
		/* CAB */
		struct dma_queue cab_queue;
		unsigned int cab_queue_len;
		uint32_t cab_flush_time;
		uint8_t cab_flush_trigger;
# define		CARL9170_CAB_TRIGGER_EMPTY		0
# define		CARL9170_CAB_TRIGGER_ARMED		1
# define		CARL9170_CAB_TRIGGER_DEFER		2
#endif /* CONFIG_CARL9170FW_CAB_QUEUE */

		/* tx status */
		uint8_t tx_status_pending;
		uint8_t tx_status_head_idx;
		uint8_t tx_status_tail_idx;
		struct carl9170_tx_status tx_status_cache[CARL9170_TX_STATUS_NUM];

#ifdef CONFIG_CARL9170FW_HANDLE_BACK_REQ
		/* BA(R) Request Handler */
		struct dma_desc *ba_desc;
		struct carl9170_bar_ctx ba_cache[CARL9170_BACK_REQS_NUM];
		uint8_t ba_desc_available;
		uint8_t ba_head_idx;
		uint8_t ba_tail_idx;
#endif /* CONFIG_CARL9170FW_HANDLE_BACK_REQ */
	} wlan;

	struct {
		uint8_t config;
		uint8_t interface_setting;
		bool ep0_stall;
		bool ep0_trigger;

		void *ep0_txrx_buffer;
		unsigned int ep0_txrx_len,
			     ep0_txrx_pos;

		struct ar9170_usb_config *cfg_desc;
		struct ar9170_usb_config *os_cfg_desc;

		/*
		 * special buffers for command & response handling
		 *
		 * the firmware uses a sort of ring-buffer to communicate
		 * to the host.
		 */
		uint8_t int_desc_available;
		uint8_t int_head_index;
		uint8_t int_tail_index;
		uint8_t int_pending;
		struct dma_desc *int_desc;
		struct ar9170_rsp int_buf[CARL9170_INT_RQ_CACHES];

#ifdef CONFIG_CARL9170FW_DEBUG_USB
		/* USB printf */
		unsigned int put_index;
		uint8_t put_buffer[CARL9170_MAX_CMD_PAYLOAD_LEN];
#endif /* CONFIG_CARL9170FW_DEBUG_USB */
	} usb;

	struct {
		/* PHY/RF state */
		uint32_t frequency;
		uint8_t dynamic_ht2040;
		uint8_t ht_enable;
		uint8_t channel_offset;
	} phy;
};

/*
 * global firmware context struct.
 *
 * NOTE: This struct will zeroed out in boot()
 */
extern struct firmware_context_struct fw;

void boot(void);
#endif	/* __AR9170_H */
