/*
 * The ar9170 firwmare gets copied into the device's
 * Program RAM (pram), which has a size of 32K, but
 * also has to accomodate the stack the device uses,
 * which starts at the top of the 32k, so we pretend
 * that we just have 16k of pram.
 *
 * This section documents some of the other areas
 * mapped into the firmware processor's address space
 * as well.
 */

ENTRY(_start);

MEMORY
{
    pram     : ORIGIN = 0x200000, LENGTH = 16k
}

SECTIONS
{
	/*
	 * The ar9170 boot code will execute the code
	 * at address 0x04 from the loaded firmware as
	 * such we must ensure our starting routine
	 * is kept at that address.
	 */
	.padding : {
		/* NOP NOP just in case */
		LONG(0x00090009)
	} > pram

	.boot     : { *(.boot) } > pram

	.text	: { *(.text*) } > pram
	.rodata	: { *(.rodata*) } > pram
	.bss	: { *(.bss) } > pram
	.data	: { *(.data*) } > pram
}
