/*
 * carl9170 firmware - used by the ar9170 wireless device
 *
 * Code to handle commands from the host driver.
 *
 * Copyright (c) 2000-2005 ZyDAS Technology Corporation
 * Copyright (c) 2007-2009 Atheros Communications, Inc.
 * Copyright	2009	Johannes Berg <johannes@sipsolutions.net>
 * Copyright	2009	Christian Lamparter <chunkeey@googlemail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "config.h"
#include "compiler.h"
#include "types.h"
#include "io.h"
#include "carl9170.h"
#include "cmd.h"
#include "cam.h"
#include "desc.h"
#include "phy.h"
#include "printf.h"
#include "time.h"
#include "wlan.h"

void handle_cmd(struct ar9170_rsp *resp)
{
	struct ar9170_cmd *cmd = &dma_mem.reserved.cmd.cmd;
	uint16_t i;

	resp->cmd = cmd->cmd;
	resp->len = cmd->len;
	resp->sequence = cmd->sequence;

	switch (cmd->cmd) {
	case AR9170_CMD_RREG:
		for (i = 0; i < cmd->len / 4; i++)
			resp->rreg_res.vals[i] = get(cmd->rreg.regs[i]);
		resp->len = cmd->len;
		break;

	case AR9170_CMD_WREG:
		resp->len = 0;
		for (i = 0; i < cmd->len / 8; i++)
			set(cmd->wreg.regs[i].addr, cmd->wreg.regs[i].val);
		break;

#ifdef CONFIG_CARL9170FW_SECURITY_ENGINE
	case AR9170_CMD_EKEY:
		resp->len = 1;
		set_key(&cmd->setkey);
		break;

	case AR9170_CMD_DKEY:
		/* Disable Key */
		resp->len = 1;
		disable_key(&cmd->disablekey);
		break;
#endif /* CONFIG_CARL9170FW_SECURIT_ENGINE */

#ifdef CONFIG_CARL9170FW_RADIO_FUNCTIONS
	case AR9170_CMD_FREQUENCY:
	case AR9170_CMD_RF_INIT:
		rf_cmd(cmd, resp);
		break;

	case AR9170_CMD_FREQ_START:
		notify_set_channel();
		resp->len = 0;
		break;
#endif /* CONFIG_CARL9170FW_RADIO_FUNCTIOS */

	case CARL9170_CMD_TX_STUCK:
		wlan_tx_stuck(cmd, resp);
		break;

	case AR9170_CMD_ECHO:
		memcpy(resp->echo.vals, cmd->echo.vals, cmd->len);
		break;

	case AR9170_CMD_SWRST:
		resp->len = 0;
		wlan_mac_reset();
		break;

	case AR9170_CMD_REBOOT:
		/*
		 * reboot does not return
		 *
		 * resp->len = 0;
		 */

		reboot();
		break;

	default:
		break;
	}
}

