/*
 * carl9170 firmware - used by the ar9170 wireless device
 *
 * Firmware header definition
 *
 * Copyright	2009	Christian Lamparter <chunkeey@googlemail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
#include "version.h"
#include "config.h"
#include "compiler.h"
#include "types.h"

#include "carl9170.h"
#include "hdr.h"
#include "desc.h"

const struct carl9170_firmware_descriptor __section(fwhdr) carl9170_fw_desc = {
	.otus = {
		FILL_DESC(OTUS_MAGIC, sizeof(struct carl9170_fw_otus_desc_v1),
			  CARL9170_FW_OTUS_DESC_MIN_VER,
			  CARL9170_FW_OTUS_DESC_CUR_VER),

		.fw_feature_set = cpu_to_le32(BIT(CARL9170_FW_DUMMY_FEATURE) |
#ifdef CONFIG_CARL9170FW_INTEGRATE_HEADER
				  BIT(CARL9170_FW_INTEGRATED_DESC_HEADER) |
#endif /* CONFIG_CARL9170FW_INTEGRATE_HEADER */
#ifdef CONFIG_CARL9170FW_RADIO_FUNCTIONS
				  BIT(AR9170_FW_COMMAND_PHY) |
#endif /* CONFIG_CARL9170FW_RADIO_FUNCTIONS */
#ifdef CONFIG_CARL9170FW_SECURITY_ENGINE
				  BIT(AR9170_FW_COMMAND_CAM) |
#endif /* CONFIG_CARL9170FW_SECURITY_ENGINE */
#ifdef CONFIG_CARL9170FW_CAB_QUEUE
				  BIT(CARL9170_FW_WLANTX_CAB) |
#endif /* CONFIG_CARL9170FW_CAB_QUEUE */
#ifdef CONFIG_CARL9170FW_DELAYED_TX
				  BIT(AR9170_FW_WLANTX_DELAYED_TX) |
#endif /* CONFIG_CARL9170FW_DELAYED_TX */
#ifdef CONFIG_CARL9170FW_HANDLE_BACK_REQ
				  BIT(AR9170_FW_HANDLE_BACK_REQ) |
#endif /* CONFIG_CARL9170FW_HANDLE_BACK_REQ */
#ifdef CONFIG_CARL9170FW_DEBUG
				  BIT(CARL9170_FW_UNUSEABLE) |
#endif /* CONFIG_CARL9170FW_DEBUG */
				  BIT(CARL9170_FW_CARL9170_V1) |
				  BIT(AR9170_FW_COMMAND_BASIC) |
				  (0)),

		.fw_year_month_day = CARL9170_FW_SET_DAY(CARL9170FW_VERSION_DAY) +
			CARL9170_FW_SET_MONTH(CARL9170FW_VERSION_MONTH) +
			CARL9170_FW_SET_YEAR(CARL9170FW_VERSION_YEAR),
		.bcn_addr = (__le32) cpu_to_le32(&dma_mem.reserved.bcn),
		.bcn_len = (__le16) cpu_to_le16(sizeof(dma_mem.reserved.bcn)),
	},

	.usb = {
		FILL_DESC(USB_MAGIC, sizeof(struct carl9170_fw_usb_desc_v1),
			  CARL9170_FW_USB_DESC_MIN_VER,
			  CARL9170_FW_USB_DESC_CUR_VER),

		.usb_feature_set = cpu_to_le32(BIT(CARL9170_FW_USB_DUMMY_FEATURE) |
				   BIT(AR9170_FW_USB_RESP_EP2) |
#ifdef CONFIG_CARL9170FW_USB_INIT_FIRMWARE
				   BIT(AR9170_FW_USB_INIT_FIRMWARE) |
# ifdef CONFIG_CARL9170FW_USB_UP_STREAM
				   BIT(AR9170_FW_USB_UP_STREAM) |
# endif /* CONFIG_CARL9170FW_USB_UP_STREAM */
# ifdef CONFIG_CARL9170FW_USB_DOWN_STREAM
				   BIT(AR9170_FW_USB_DOWN_STREAM) |
# endif /* CONFIG_CARL9170FW_USB_DOWN_STREAM */
#endif /* CONFIG_CARL9170FW_USB_INIT_FIRMWARE */
				   (0)),

		.miniboot_size = cpu_to_le16(0),
		.tx_descs = AR9170_TX_BLOCK_NUMBER,
		.rx_max_frame_len = cpu_to_le16(CONFIG_CARL9170FW_RX_FRAME_LEN),
		.tx_frag_len = cpu_to_le16(AR9170_BLOCK_SIZE),
	},

#ifdef CONFIG_CARL9170FW_HEADER_MOTD
	.motd = {
		FILL_DESC(MOTD_MAGIC, sizeof(struct carl9170_fw_motd_desc_v1),
			  CARL9170_FW_MOTD_DESC_MIN_VER,
			  CARL9170_FW_MOTD_DESC_CUR_VER),
		.desc = "Community AR9170 Linux",
		.release = CARL9170FW_VERSION_GIT,
	},
#endif /* CONFIG_CARL9170FW_HEADER_MOTD */

#ifdef CONFIG_CARL9170FW_DEBUG
	.debug = {
		FILL_DESC(DBG_MAGIC, sizeof(struct carl9170_fw_debug_desc_v1),
			  CARL9170_FW_DEBUG_DESC_MIN_VER,
			  CARL9170_FW_DEBUG_DESC_CUR_VER),
	},
#endif /* CONFIG_CARL9170FW_DEBUG */

	.last = {
		FILL_DESC(LAST_MAGIC, sizeof(struct carl9170_fw_last_desc_v1),
			  CARL9170_FW_LAST_DESC_MIN_VER,
			  CARL9170_FW_LAST_DESC_CUR_VER),
	},
};
